;------------------------------------------------------------;
; The author, Philippe Coulombe, has given permission to use ;
; this module to the users of The GeneSys Development System ;
; and is for educational use only. Users are not allowed to  ;
; use this module in commercial software. To use this module ;
; for any for profit purpose, the author must be contacted.  ;
; Send your request directly to Philippe, ebmoluocp@yahoo.ca ;
;------------------------------------------------------------;
; Displays a Browse For Folder dialog. For a basic dialog,   ;
; all parameters can be NULL except lpBuffer                 ;
;                                                            ;
; Returns the lpBuffer pointer in EAX if successful or NULL  ;
; otherwise.                                                 ;
;------------------------------------------------------------;

.586

.MODEL FLAT, STDCALL

OPTION CASEMAP:NONE


INCLUDE \GeneSys\include\windows.inc
INCLUDE \GeneSys\include\ole32.inc
INCLUDE \GeneSys\include\shell32.inc
INCLUDE \GeneSys\include\user32.inc


CenterWindow                 PROTO :DWORD, :DWORD


.DATA?

ALIGN 4

lpszInitialPath              DWORD ?
lpszCaption                  DWORD ?
lpPrevWndProc                DWORD ?


.CODE

;
; lpBuffer        : MAX_PATH length buffer to get user selection and internal use (Required)
; hWndOwner       : Owner window handle (NULL / Handle)
; lpszText        : Text above tree view (NULL / String pointer)
; lpszCaption     : Caption text (NULL / String pointer)
; dwCSIDL         : CSIDL constants (NULL / CSIDL constants)
; lpszInitPath    : Pre-selected folder path (NULL / String pointer)
; bNewFldrBtn     : "New Folder" button (TRUE / FALSE)
;
BrowseForFolderA PROC USES ebx p_lpBuffer:DWORD, p_hWndOwner:DWORD, p_lpszText:DWORD, p_lpszCaption:DWORD, \
                               p_dwCSIDL:DWORD, p_lpszInitPath:DWORD, p_bNewFldrBtn:DWORD

   LOCAL l_lpIDL:DWORD
   LOCAL l_bi:BROWSEINFO

   mov ebx, NULL   ; returned value if cancel/fail

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ; Fill BROWSEINFO structure
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

   mov eax, p_hWndOwner
   mov l_bi.hwndOwner, eax

   invoke SHGetSpecialFolderLocation, p_hWndOwner, p_dwCSIDL, addr l_lpIDL
   .if (eax == S_OK)
       mov eax, l_lpIDL
   .else
       mov eax, NULL
   .endif
   mov l_bi.pidlRoot, eax

   mov l_bi.pszDisplayName, NULL

   mov eax, p_lpszText
   mov l_bi.lpszTitle, eax

   mov eax, (BIF_RETURNONLYFSDIRS or BIF_NEWDIALOGSTYLE)
   .if (p_bNewFldrBtn == FALSE)
       or eax, BIF_NONEWFOLDERBUTTON
   .endif
   mov l_bi.ulFlags, eax

   mov l_bi.lpfn, offset BrowseCallbackProc

   mov eax, p_lpBuffer
   mov l_bi.lParam, eax

   mov l_bi.iImage, NULL

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ; Dialog caption and initial path used by BrowseCallbackProc
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

   mov eax, p_lpszInitPath
   mov lpszInitialPath, eax

   mov eax, p_lpszCaption
   mov lpszCaption, eax

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ; Display the dialog and get user selection through lpIDL
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

   invoke SHBrowseForFolderA, addr l_bi
   mov l_lpIDL, eax

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ; pIDLs are freed and lpBuffer is filled with user selection
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

   .if (l_bi.pidlRoot != NULL)
       invoke CoTaskMemFree, l_bi.pidlRoot
   .endif

   .if (l_lpIDL != NULL)
       invoke SHGetPathFromIDListA, l_lpIDL, p_lpBuffer   ; Return FALSE on virtual folder
       .if (eax == TRUE)
           mov ebx, p_lpBuffer   ; success
       .endif
       invoke CoTaskMemFree, l_lpIDL
   .endif

   mov eax, ebx
   ret

BrowseForFolderA ENDP

ALIGN 4
;
; lParam = Dialog window handle
; lpData = Pointer to a MAX_PATH buffer (bi.lParam)
;
BrowseCallbackProc PROC PRIVATE p_hwnd:DWORD, p_uMsg:DWORD, p_lParam:DWORD, p_lpData:DWORD

   mov eax, p_uMsg

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ; Disable OK button on virtual folder
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

   .if (eax == BFFM_SELCHANGED)

       invoke SHGetPathFromIDListA, p_lParam, p_lpData          ; Return FALSE on virtual folder
       invoke SendMessageA, p_hwnd, BFFM_ENABLEOK, NULL, eax

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ; Set dialog caption text and default path
   ; Use WindowProc for dialog centering
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

   .elseif (eax == BFFM_INITIALIZED)

       .if (lpszCaption != NULL)
           invoke SendMessageA, p_hwnd, WM_SETTEXT, NULL, lpszCaption
       .endif

       .if (lpszInitialPath != NULL)
           invoke SendMessageA, p_hwnd, BFFM_SETSELECTIONA, TRUE, lpszInitialPath
       .endif

       push offset WindowProc
       push GWL_WNDPROC
       push p_hwnd
       call SetWindowLongA
       mov lpPrevWndProc, eax

   .endif

   xor eax, eax
   ret

BrowseCallbackProc ENDP

ALIGN 4
;
; Used to center dialog into owner window
;
WindowProc PROC PRIVATE p_hWnd:DWORD, p_uMsg:DWORD, p_wParam:DWORD, p_lParam:DWORD

   .if (p_uMsg == WM_SHOWWINDOW)

       invoke GetParent, p_hWnd
       invoke CenterWindow, p_hWnd, eax

       invoke SetWindowLongA, p_hWnd, GWL_WNDPROC, lpPrevWndProc   ; restore previous window proc

   .endif

   invoke CallWindowProcA, lpPrevWndProc, p_hWnd, p_uMsg, p_wParam, p_lParam

   ret

WindowProc ENDP

END
