;------------------------------------------------------------;
; The author, Philippe Coulombe, has given permission to use ;
; this module to the users of The GeneSys Development System ;
; and is for educational use only. Users are not allowed to  ;
; use this module in commercial software. To use this module ;
; for any for profit purpose, the author must be contacted.  ;
; Send your request directly to Philippe, ebmoluocp@yahoo.ca ;
;------------------------------------------------------------;
; Center a child window hWndChild into a parent window or    ;
; the desktop if hWndParent is set to NULL. The parent does  ;
; not need to be the owner. A child window cannot be moved   ;
; outside the desktop workarea.                              ;
;                                                            ;
; Returns non-zero if the child window has been centered.    ;
;------------------------------------------------------------;

.586

.MODEL FLAT, STDCALL

OPTION CASEMAP:NONE


INCLUDE \GeneSys\include\windows.inc
INCLUDE \GeneSys\include\user32.inc


.CODE

CenterWindow PROC p_hWndChild:DWORD, p_hWndParent:DWORD

   LOCAL l_rectChild:RECT         ; Child window coordonate
   LOCAL l_rectParent:RECT        ; Parent window coordonate
   LOCAL l_rectDesktop:RECT       ; Desktop coordonate (WORKAREA)

   invoke GetWindowRect, p_hWndChild, addr l_rectChild
   .if (eax != 0)    ; 0 = no centering possible

       invoke SystemParametersInfoA, SPI_GETWORKAREA, NULL, addr l_rectDesktop, NULL
       .if (eax != 0)    ; 0 = no centering possible

           invoke GetWindowRect, p_hWndParent, addr l_rectParent
           .if (eax == 0)    ; 0 = we take the desktop as parent (invalid or NULL hWndParent)

               mov eax, l_rectDesktop.left
               mov l_rectParent.left, eax
               mov eax, l_rectDesktop.top
               mov l_rectParent.top, eax
               mov eax, l_rectDesktop.right
               mov l_rectParent.right, eax
               mov eax, l_rectDesktop.bottom
               mov l_rectParent.bottom, eax

           .endif

           ;
           ; edi,esi : child window new x,y coordonate
           ;
           push edi
           push esi
           ;
           ; Get new coordonate and make sure the child window
           ; is not moved outside the desktop workarea
           ;
           mov eax, l_rectChild.right
           sub eax, l_rectChild.left                      ;
           mov ecx, eax                                   ; ecx = child width = right - left
           mov eax, l_rectParent.right
           sub eax, l_rectParent.left
           sub eax, ecx                                   ; eax = Parent width - Child width...
           sar eax, 1                                     ; divided by 2
           add eax, l_rectParent.left                     ; eax = temporary left coord (need validation)
           .if (sdword ptr eax < l_rectDesktop.left)
               mov eax, l_rectDesktop.left
           .endif
           mov edi, eax
           add eax, ecx                                   ; eax = new left coord + child width
           .if (sdword ptr eax > l_rectDesktop.right)     ; if child right outside desktop workarea...
               mov edi, l_rectDesktop.right
               sub edi, ecx                               ; right = desktop right - child width
           .endif

           mov eax, l_rectChild.bottom
           sub eax, l_rectChild.top                       ;
           mov ecx, eax                                   ; ecx = child height = bottom - top
           mov eax, l_rectParent.bottom
           sub eax, l_rectParent.top
           sub eax, ecx                                   ; eax = Parent height - Child height...
           sar eax, 1
           add eax, l_rectParent.top
           .if (sdword ptr eax < l_rectDesktop.top)       ; eax (child top) must not be smaller, if so...
               mov eax, l_rectDesktop.top                 ; child top = Desktop.top
           .endif
           mov esi, eax
           add eax, ecx                                   ; eax = new top coord + child height
           .if (sdword ptr eax > l_rectDesktop.bottom)
               mov esi, l_rectDesktop.bottom              ; child is outside desktop bottom
               sub esi, ecx                               ; child top = Desktop.bottom - child height
           .endif

           ;
           ; Now we have the new coordonate - the dialog window can be moved
           ;
           invoke SetWindowPos, p_hWndChild, NULL, edi, esi, NULL, NULL,
                               (SWP_NOSIZE or SWP_NOZORDER or SWP_NOCOPYBITS)
           pop esi
           pop edi

           ret

       .endif

   .endif

   xor eax, eax
   ret

CenterWindow ENDP

END
