;------------------------------------------------------------;
; The author, Philippe Coulombe, has given permission to use ;
; this module to the users of The GeneSys Development System ;
; and is for educational use only. Users are not allowed to  ;
; use this module in commercial software. To use this module ;
; for any for profit purpose, the author must be contacted.  ;
; Send your request directly to Philippe, ebmoluocp@yahoo.ca ;
;------------------------------------------------------------;
;         Return TRUE if lpszFilePath is a valid PE          ;
;------------------------------------------------------------;

.586

.MODEL FLAT, STDCALL

OPTION CASEMAP:NONE


INCLUDE \GeneSys\include\windows.inc
INCLUDE \GeneSys\include\kernel32.inc


.CODE

OPTION PROLOGUE:NONE
OPTION EPILOGUE:NONE

IsPeFileW PROC p_lpszFilePath:DWORD

   LOCAL l_bReturn:DWORD
   LOCAL l_hFile:DWORD
   LOCAL l_hFileMap:DWORD
   LOCAL l_lpFileMap:DWORD
   LOCAL l_dwFileLen:DWORD

   mov l_bReturn, FALSE

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ; Get PE length (bytes) and file pointer (mapping)
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

   mov eax, [esp+4]
   invoke CreateFileW, eax, GENERIC_READ, NULL, NULL, OPEN_EXISTING, NULL, NULL
   cmp eax, INVALID_HANDLE_VALUE
   je Exit

   mov l_hFile, eax

   invoke GetFileSize, eax, NULL
   cmp eax, 1024     ; 1024 = minimum bytes length for a PE file
   jnge Free_hFile

   mov l_dwFileLen, eax

   invoke CreateFileMappingW, l_hFile, NULL, PAGE_READONLY, NULL, NULL, NULL
   test eax, eax
   jz Free_hFile

   mov l_hFileMap, eax

   invoke MapViewOfFile, eax, FILE_MAP_READ, NULL, NULL, NULL
   test eax, eax
   jz Free_hFileMap

   mov l_lpFileMap, eax

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ; Check predefined values of PE file
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

   cmp word ptr [eax.IMAGE_DOS_HEADER].e_magic, IMAGE_DOS_SIGNATURE
   jne Free_lpFileMap

   add eax, dword ptr [eax.IMAGE_DOS_HEADER].e_lfanew
   mov edx, sizeof IMAGE_NT_HEADERS
   add edx, eax
   sub edx, l_lpFileMap
   cmp edx, l_dwFileLen
   jnle Free_lpFileMap

   cmp dword ptr [eax.IMAGE_NT_HEADERS].Signature, IMAGE_NT_SIGNATURE
   jne Free_lpFileMap

   cmp word ptr [eax.IMAGE_NT_HEADERS].OptionalHeader.Magic, IMAGE_NT_OPTIONAL_HDR32_MAGIC
   jne Free_lpFileMap

   mov l_bReturn, TRUE

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ; Close objects and set the returned value
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Free_lpFileMap:

   invoke UnmapViewOfFile, l_lpFileMap

Free_hFileMap:

   invoke CloseHandle, l_hFileMap

Free_hFile:

   invoke CloseHandle, l_hFile

Exit:

   mov eax, l_bReturn
   ret 4

IsPeFileW ENDP

OPTION PROLOGUE:PROLOGUEDEF
OPTION EPILOGUE:EPILOGUEDEF

END
